/*  $Id: voice_decoder.h,v 1.11 2004/01/02 15:42:45 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

//! clanVoice="Encoding"
//! header=voice.h

#ifndef header_voice_decoder
#define header_voice_decoder

#if _MSC_VER > 1000
#pragma once
#endif

#include "../Sound/soundprovider_session.h"
#include <string>

class CL_VoiceDecoder_Generic;

//: Voice decoder
//- !group=Voice/Encoding!
//- !header=voice.h!
class CL_VoiceDecoder : public CL_SoundProvider_Session
{
//! Construction:
public:
	//: Construct voice decoder.
	CL_VoiceDecoder();

	~CL_VoiceDecoder();

//! Attributes:
public:
	//: Returns the number of samples in the soundbuffer.
	virtual int get_num_samples() const;

	//: Returns the playback frequency of the input data.
	//return: Playback frequency.
	virtual int get_frequency() const;

	//: Returns the current position in the playback stream.
	virtual int get_position() const;

	//: Returns the playback sample format.
	//return: The playback Sample format.
	virtual CL_SoundFormat get_format() const;

	//: Returns the number of channels filled when get_data is called.
	virtual int get_num_channels() const;

//! Operations:
public:
	// Decodes packet and makes it available through the stream provider session.
	void decode_packet(const std::string &packet);

	//: Returns true if no more input data is available.
	//return: True if end of input data. False otherwise.
	virtual bool eof() const;
	
	//: Stops the current stream.
	virtual void stop();

	//: Start/continue playing of the stream.
	//return: True for success. False otherwise.
	virtual bool play();
		
	//: Sets the position within the current stream.
	//param pos: Position to seek to.
	//return: True for success. False otherwise.
	virtual bool set_position(int pos);
	
	//: Called when a playback session needs more sample data.
	//param data_ptr: Points to a buffer that should be filled with sample data.
	//param data_requested: Samples of data requested.
	//return: Number of samples actually filled with sample data.
	virtual int get_data(void **data_ptr, int data_requested);

//! Implementation:
private:
	CL_VoiceDecoder_Generic *impl;
};

#endif
