/*  $Id: inputsource_memory.cpp,v 1.7 2003/07/20 22:30:53 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "Core/precomp.h"
#include <API/Core/System/cl_assert.h>
#include <API/Core/System/clanstring.h>
#include <API/Core/IOData/inputsource_memory.h>

CL_InputSource_Memory::CL_InputSource_Memory(std::string _data)
{
	data = _data;
	pos = 0;
}

CL_InputSource_Memory::~CL_InputSource_Memory()
{
}

int CL_InputSource_Memory::read(void *dest_data, int size)
{
	cl_assert(pos + size <= data.size());
	
	memcpy(dest_data, data.data() + pos, size);
	pos += size;

	return size;
}

void CL_InputSource_Memory::open()
{
	pos = 0;
}

void CL_InputSource_Memory::close()
{
}

CL_InputSource *CL_InputSource_Memory::clone() const
{
	return new CL_InputSource_Memory(data);
}

int CL_InputSource_Memory::tell() const
{
	return pos;
}

void CL_InputSource_Memory::seek(int new_pos, SeekEnum seek_type)
{
	switch (seek_type)
	{
	case seek_set:
		pos = new_pos;
		break;
	
	case seek_cur:
		pos += new_pos;
		break;
	
	case seek_end:
		pos = data.size() + new_pos;
		break;
	
	default: // invalid seek type!
		cl_assert(false);
	}
}

int CL_InputSource_Memory::size() const
{
	return data.size();
}

void CL_InputSource_Memory::push_position()
{
	cl_assert(false); // not implemented yet.
}

void CL_InputSource_Memory::pop_position()
{
	cl_assert(false); // not implemented yet.
}

std::string CL_InputSource_Memory::get_data() const
{
	return data;
}

