/*  $Id: menu_node_boring.cpp,v 1.2 2004/01/03 23:03:22 sphair Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "API/Core/Resources/resource_manager.h"
#include "API/Core/Math/rect.h"
#include "API/Display/surface.h"
#include "API/Core/XML/dom_element.h"
#include "API/Display/display.h"
#include "API/GUIStyleBoring/stylemanager_boring.h"
#include "API/GUI/menu.h"
#include "API/GUIStyleBoring/menu_node_boring.h"

CL_MenuNode_Boring::CL_MenuNode_Boring(
	CL_MenuNode *node,
	CL_StyleManager_Boring *style)
:
	CL_ComponentStyle(node),
	node(node),
	style(style),
	resources(0),
	arrow(0)
{
	resources = style->get_resources();

	if (resources->exists("Menu/submenu_arrow"))
		arrow = new CL_Surface("Menu/submenu_arrow", resources);	
	
	slots.connect(node->sig_set_options(), this, &CL_MenuNode_Boring::on_set_options);
	slots.connect(node->sig_paint(), this, &CL_MenuNode_Boring::on_paint);
	slots.connect(node->sig_reposition(), this, &CL_MenuNode_Boring::on_reposition); // todo: sync names?
	slots.connect(node->sig_get_preferred_size(), this, &CL_MenuNode_Boring::on_get_preferred_size);
}

CL_MenuNode_Boring::~CL_MenuNode_Boring()
{
	delete arrow;
}

void CL_MenuNode_Boring::on_set_options(const CL_DomElement &options)
{
	// ?
}

void CL_MenuNode_Boring::on_paint()
{
	CL_Rect area(1,0,node->get_width()-1,node->get_height());

	// mouseover fill
	if( node->has_mouse_over() )
		CL_Display::fill_rect( area, 
			CL_Gradient(
				CL_Color(255,255,255,128), CL_Color(254, 207, 147, 128),
				CL_Color(255,255,255,128), CL_Color(254, 207, 147, 128) ));

	if( arrow && node->has_submenu() )
	{		
		if( node->get_parent_menu()->is_vertical() ) // no arrow in horizontal menus
			arrow->draw( area.right-8, 4 );
	}
}

void CL_MenuNode_Boring::on_reposition()
{
	// get screen rect and resize submenus according to that.
	CL_Rect sr = node->get_screen_rect();
	
	CL_Menu *submenu = node->get_submenu();

	if( submenu )
	{
		if( node->get_parent_menu()->is_vertical() )
			submenu->reposition( sr.right, sr.top+2 );
		else
			submenu->reposition( sr.left, sr.bottom );
	}
}

void CL_MenuNode_Boring::on_get_preferred_size(CL_Size &size)
{
	if( size.width < 220 )
		size.width = 220;
}
