/*  $Id: stylemanager_boring.cpp,v 1.5 2004/01/05 15:46:25 sphair Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "API/Display/display.h"
#include "API/Display/surface.h"
#include "API/Core/Resources/resource_manager.h"
#include "API/GUI/component_manager.h"
#include "API/guistyleboring.h"

// Construction:

CL_StyleManager_Boring::CL_StyleManager_Boring(CL_ResourceManager *resources)
: CL_StyleManager(resources)
{
}

CL_StyleManager_Boring::~CL_StyleManager_Boring()
{
}

void CL_StyleManager_Boring::get_color(
	GUIColor col,
	int &r,
	int &g,
	int &b,
	int &a)
{
	a = 255;

	switch (col)
	{
	case GUICOLOR_SELECTED_OUTLINE:
	case GUICOLOR_DARK_OUTLINE:
		r = 128;
		g = 128;
		b = 128;
		break;
	case GUICOLOR_BRIGHT_SHADE:
		r = 255;
		g = 255;
		b = 255;
		break;
	case GUICOLOR_MEDIUM_SHADE:
		r = 211;
		g = 208;
		b = 200;
		break;
	case GUICOLOR_DARKER_SHADE:
		r = 65;
		g = 65;
		b = 65;
		break;
	case GUICOLOR_DARK_SHADE:
		r = 188;
		g = 190;
		b = 184;
		break;
	case GUICOLOR_DARK_OUTLINE_DISABLED:
		r = 192;
		g = 194;
		b = 189;
		break;
	case GUICOLOR_BRIGHT_SHADE_DISABLED:
		r = 243;
		g = 245;
		b = 241;
		break;
	case GUICOLOR_DARK_SHADE_DISABLED:
		r = 226;
		g = 227;
		b = 222;
		break;
	case GUICOLOR_WHITE:
		r = 255;
		g = 255;
		b = 255;
		break;
	case GUICOLOR_RED:
		r = 255;
		g = 55;
		b = 55;
		break;
	case GUICOLOR_CARET:
		r = 0;
		g = 0;
		b = 0;
		break;
	case GUICOLOR_PROGRESSBAR_FILLED:
	case GUICOLOR_SELECTION:
		r = 48;
		g = 105;
		b = 197;
		break;
	case GUICOLOR_PROGRESSBAR:
	case GUICOLOR_SCROLLBAR:
		r = 196;
		g = 197;
		b = 190;
		break;
	case GUICOLOR_BUTTON:
		r = 236;
		g = 238;
		b = 232;
		break;
	case GUICOLOR_FOCUS:
		r = 236;
		g = 238;
		b = 100;
		break;
	case GUICOLOR_BUTTON_DISABLED:
		r = 232;
		g = 235;
		b = 230;
		break;
	case GUICOLOR_BUTTON_TOGGLED:
		r = 230;
		g = 231;
		b = 225;
		break;
	case GUICOLOR_WINDOW_NORMAL:
		r = 235;
		g = 235;
		b = 227;
		break;
	case GUICOLOR_WINDOW_TITLEBAR:
		r = 40;
		g = 90;
		b = 150;
		break;
	case GUICOLOR_WINDOW_TITLEBAR_DISABLED:
		r = 40;
		g = 40;
		b = 40;
		break;
	default:
		r = 255;
		g = 255;
		b = 255;
	}
}

void CL_StyleManager_Boring::fill_rect(int x1, int y1, int x2, int y2, GUIColor col)
{
	int r, g, b, a;
	get_color(col, r, g, b, a);

	CL_Display::fill_rect(CL_Rect(x1, y1, x2, y2), CL_Color(r, g, b));
}

void CL_StyleManager_Boring::draw_rect(int x1, int y1, int x2, int y2, GUIColor col)
{
	int r, g, b, a;
	get_color(col, r, g, b, a);

	CL_Display::draw_rect(CL_Rect(x1, y1, x2, y2), CL_Color(r, g, b));
}

void CL_StyleManager_Boring::draw_line(int x1, int y1, int x2, int y2, GUIColor col)
{
	int r, g, b, a;
	get_color(col, r, g, b, a);

	CL_Display::draw_line(x1, y1, x2, y2, CL_Color(r, g, b, a));
}

void CL_StyleManager_Boring::draw_box(int x1, int y1, int x2, int y2, GUIColor topleft, GUIColor bottomright)
{
	int r, g, b, a;
	get_color(topleft, r, g, b, a);

	// Top line
	CL_Display::draw_line(x1, y1, x2 - 1, y1, CL_Color(r, g, b, a));

	// Left line
	CL_Display::draw_line(x1, y1 + 1, x1, y2 - 1, CL_Color(r, g, b, a));
	
	get_color(bottomright, r, g, b, a);

	// Bottom line
	CL_Display::draw_line(x1 + 1, y2 - 1, x2 - 1, y2 - 1, CL_Color(r, g, b, a));

	// Right line
	CL_Display::draw_line(x2 - 1, y1 + 1, x2 - 1, y2, CL_Color(r, g, b, a));
}

// Overridables:

void CL_StyleManager_Boring::connect_styles(
	const std::string &type,
	CL_Component *component)
{
	if (type == "label")
		component->set_style(new CL_Label_Boring((CL_Label *) component, this));

	else if (type == "button")
		component->set_style(new CL_Button_Boring((CL_Button *) component, this));

	else if (type == "frame")
		component->set_style(new CL_Frame_Boring((CL_Frame *) component, this));

	else if (type == "image")
		component->set_style(new CL_Image_Boring((CL_Image *) component, this));

	else if (type == "inputbox")
		component->set_style(new CL_InputBox_Boring((CL_InputBox *) component, this));

	else if (type == "scrollbar")
		component->set_style(new CL_ScrollBar_Boring((CL_ScrollBar *) component, this));

	else if (type == "progressbar")
		component->set_style(new CL_ProgressBar_Boring((CL_ProgressBar *) component, this));

	else if (type == "radiobutton")
		component->set_style(new CL_RadioButton_Boring((CL_RadioButton *) component, this));

	else if (type == "checkbox")
		component->set_style(new CL_CheckBox_Boring((CL_CheckBox *) component, this));

	else if (type == "listbox")
		component->set_style(new CL_ListBox_Boring((CL_ListBox *) component, this));

	else if (type == "menu")
		component->set_style(new CL_Menu_Boring((CL_Menu *) component, this));

	else if (type == "menu_node")
		component->set_style(new CL_MenuNode_Boring((CL_MenuNode *) component, this));

	else if (type == "menu_item")
		component->set_style(new CL_MenuItem_Boring((CL_MenuItem *) component, this));

//	else if (type == "treeview")
//		component->set_style(new CL_TreeView_Boring((CL_TreeView *) component, this));

//	else if (type == "treeview_item")
//		component->set_style(new CL_TreeItem_Boring((CL_TreeItem *) component, this));

	else if (type == "window")
		component->set_style(new CL_Window_Boring((CL_Window *) component, this));
}
