/*  $Id: window_boring.cpp,v 1.4 2003/09/08 15:06:24 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "API/Display/display.h"
#include "API/Display/font.h"
#include "API/GUIStyleBoring/stylemanager_boring.h"
#include "API/GUIStyleBoring/window_boring.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_Window_Boring::CL_Window_Boring(
	CL_Window *window,
	CL_StyleManager_Boring *style) 
:
	CL_ComponentStyle(window),
	window(window),
	style(style),
	titlebar_height(21)
{
	resources = style->get_resources();
	
	font = new CL_Font("Window/font", resources);
	font_disabled = new CL_Font("Window/font_disabled", resources);
	
	// Make the window movable by dragging titlebar
	move_handler = new CL_ComponentMoveHandler(window);

	// Make the window resizable by dragging corner
	resize_handler = new CL_ComponentResizeHandler(window);

	// Create a close button on the titlebar
	button_close = new CL_Button("X", window, style);

	slots.connect(button_close->sig_clicked(), this, &CL_Window_Boring::on_close);
	slots.connect(window->sig_paint(), this, &CL_Window_Boring::on_paint);
	slots.connect(window->sig_resize(), this, &CL_Window_Boring::on_resize);

	on_resize(0,0);
}

CL_Window_Boring::~CL_Window_Boring()
{
	delete font;
	delete font_disabled;
	delete resize_handler;
	delete move_handler;
	delete button_close;
}

/////////////////////////////////////////////////////////////////////////////
// Callbacks:

void CL_Window_Boring::on_paint()
{
	int width = window->get_width();
	int height = window->get_height();
	
	style->draw_rect(0, 0, width, height, GUICOLOR_DARK_OUTLINE);

	style->draw_box(1, 1, width - 1, titlebar_height, GUICOLOR_BRIGHT_SHADE, GUICOLOR_DARK_SHADE);
	style->draw_box(1, titlebar_height, width - 1, height - 1, GUICOLOR_BRIGHT_SHADE, GUICOLOR_DARK_SHADE);

	if(window->is_enabled() == false)
		style->fill_rect(2, 2, width - 2, titlebar_height - 1, GUICOLOR_WINDOW_TITLEBAR_DISABLED);
	else
		style->fill_rect(2, 2, width - 2, titlebar_height - 1, GUICOLOR_WINDOW_TITLEBAR);
	style->fill_rect(2, titlebar_height + 1, width - 2, height - 2, GUICOLOR_WINDOW_NORMAL);

	int y = (titlebar_height + 2 - font->get_height(window->get_title())) / 2;

	if(window->is_enabled() == false)
		font_disabled->draw(7, y, window->get_title());
	else
		font->draw(7, y, window->get_title());
}

void CL_Window_Boring::on_close()
{
	if (window->is_modal())
		window->quit();
	else
		window->close();
}

void CL_Window_Boring::on_resize(int old_width, int old_height)
{
	CL_Rect client_rect(3, 24, window->get_width() - 3, window->get_height() - 3);
	window->get_client_area()->set_position(client_rect);

	CL_Rect close_rect(window->get_width() - 19, 3, window->get_width() - 3, 19);
	button_close->set_position(close_rect);

	CL_Rect move_rect(0, 0, window->get_width() - 19, titlebar_height);
	move_handler->set_position(move_rect);
	
	CL_Rect resize_rect(window->get_width() - 3, window->get_height() - 3, window->get_width(), window->get_height());
	resize_handler->set_position(resize_rect);
}
