/*  $Id: browse_master_generic.h,v 1.9 2003/10/14 15:52:13 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#ifndef header_browse_master_generic
#define header_browse_master_generic

#if _MSC_VER > 1000
#pragma once
#endif

#include "API/Network/Socket/socket.h"
#include "API/Network/Socket/ip_address.h"
#include "API/Core/System/thread.h"
#include "API/Core/System/event_trigger.h"
#include "API/Core/System/mutex.h"

#include <string>
#include <map>

class CL_BrowseMaster_Generic;
class CL_ServerDatabase;

class CL_BrowseConnection : public CL_Runnable
{
public:
	CL_BrowseConnection(CL_Socket socket, CL_BrowseMaster_Generic *doc);
	virtual ~CL_BrowseConnection() { }

	CL_Thread &get_thread() { return thread; }

private:
	virtual void run();

	void run_server();

	void run_client();

	CL_Thread thread;
	CL_Socket socket;

	std::string app_id;
	CL_ServerDatabase *database;
	CL_BrowseMaster_Generic *doc;
};

class CL_BrowseMaster_Generic
{
//! Construction:
public:
	CL_BrowseMaster_Generic();
	
	virtual ~CL_BrowseMaster_Generic();

//! Attributes:
public:

//! Operations:
public:
	void run();

	//: Setup port accepting browse clients and browse servers.
	void start_listen(const CL_IPAddress &bind_address);

	//: Stop listening.
	void stop_listen();

	//: Find the database for the application ID.
	CL_ServerDatabase *get_database(const std::string &app_id);

	//: Releases the database.
	//- <p>Must be called when done using database after a get_database call.</p>
	void release_database(const std::string &app_id);

//! Implementation:
private:
	CL_Mutex mutex;

	CL_Socket accept_socket;

	std::map<std::string, CL_ServerDatabase *> databases;
};

#endif
