/*  $Id: browse_server_generic.h,v 1.9 2003/10/14 15:52:13 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#ifndef header_browse_server_generic
#define header_browse_server_generic

#if _MSC_VER > 1000
#pragma once
#endif

#include <string>
#include "API/Network/Socket/socket.h"
#include "API/Network/Socket/ip_address.h"
#include "API/Network/NetSession/netpacket.h"

class CL_BrowseServer_Generic
{
//! Construction:
public:
	//: Setup a browsing server for an application with the specified identifier
	//: and the game server running on the specified server address.
	CL_BrowseServer_Generic(
		const std::string &app_id,
		const CL_IPAddress &server_address);
	
	//: Disconnect from browse master.
	virtual ~CL_BrowseServer_Generic();

//! Attributes:
public:

//! Operations:
public:
	//: Change the server description on the browse master.
	void set_description(const CL_NetPacket &description);

	//: Connect to browser master and register ourselves.
	void connect(const CL_IPAddress &browse_master);

	//: Listen for local area network browse queries on the specified port.
	void listen(const std::string &port);

//! Implementation:
private:
	std::string app_id;
	CL_NetPacket description;
	CL_IPAddress server_address;
	CL_Socket sock;

	//: udp socket for listening to broadcast
	CL_Socket sock_udp;
	//: Trigger slot to recive broadcast message
	CL_Slot slot_recv;
	//: method called when recived a broadcast message
	void on_udp_recv();

	bool connected;

};

#endif
