/*  $Id: surface_sdl.h,v 1.6 2003/10/14 15:52:13 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#ifndef header_surface_sdl
#define header_surface_sdl

#if _MSC_VER > 1000
#pragma once
#endif

#include "Display/surface_generic.h"
#include "Display/surface_target.h"
#include "API/Display/surface.h"
#include "API/Core/Math/size.h"

#include <SDL/SDL.h>

class CL_Surface_SDL : public CL_Surface_Target
{
//! Construction:
public:
	CL_Surface_SDL(
		CL_PixelBuffer *provider,
		bool delete_provider,
		CL_Surface::Hint hint);

	virtual ~CL_Surface_SDL();

//! Attributes:
public:
	//: Returns the width.
	virtual int get_width() const;
	
	//: Returns the height.
	virtual int get_height() const;

	//: Returns a pixelbuffer object for accessing surface data.
	virtual CL_PixelBuffer get_pixeldata();
	
	//: Returns a graphic context object for rendering to this surface.
	virtual CL_GraphicContext *get_gc();

//! Operations:
public:
	//: Uploads pixel data to surface.
	//!param pos: Position on surface where pixel data should be put.
	//!param src_rect: Part of pixelbuffer that should be uploaded.
	//!param pixeldata: CL_PixelBuffer containing the image to be uploaded.
	virtual void set_pixeldata(const CL_Point &pos, const CL_Rect &src_rect, CL_PixelBuffer &pixeldata);

	//: Draw surface on screen.
	virtual void draw(
		CL_Surface_Generic *attributes,
		int x, int y,
		CL_GraphicContext *gc);

	//: Draw surface on screen.
	virtual void draw(
		CL_Surface_Generic *attributes,
		const CL_Rect &dest,
		CL_GraphicContext *gc);

	//: Draw surface on screen.
	virtual void draw(
		CL_Surface_Generic *attributes,
		const CL_Rect &src,
		const CL_Rect &dest,
		CL_GraphicContext *gc);

//! Implementation:
private:
	CL_Sizef screen_size;

	SDL_Surface *surface;
};

#endif
